import { MinecraftUtils } from "..";
import WebRequest from "../common/WebRequest";
import { CachedPlayer } from "../types/cache/cachedPlayer";
import { CachedUsernameToUuid } from "../types/cache/cachedUsernameToUuid";

export default class PlayerTools {
  public playerEndpoint = MinecraftUtils.API_ENDPOINT + "/player/{id}";
  public playerUsernameToUuidEndpoint = MinecraftUtils.API_ENDPOINT + "/player/uuid/{id}";
  public playerSkinPartEndpoint = MinecraftUtils.API_ENDPOINT + "/player/{part}/{id}";

  /**
   * Gets information about a Minecraft player.
   *
   * @param id the id of the player
   * @returns the player information, or null if the player does not exist
   */
  public getPlayer(id: string): Promise<CachedPlayer> {
    return new Promise(async (resolve, reject) => {
      const url = this.playerEndpoint.replace("{id}", id);

      const response = await WebRequest.get(url);
      const data = response.data;

      if (response.status !== 200) {
        reject(null);
        return;
      }
      resolve(data);
    });
  }

  /**
   * Gets the UUID of a Minecraft player.
   *
   * @param id the id of the player
   * @returns the player's UUID, or null if the player does not exist
   */
  public getPlayerUuid(id: string): Promise<CachedUsernameToUuid> {
    return new Promise(async (resolve, reject) => {
      const url = this.playerUsernameToUuidEndpoint.replace("{id}", id);

      const response = await WebRequest.get(url);
      const data = response.data;

      if (response.status !== 200) {
        reject(null);
        return;
      }
      resolve(data);
    });
  }

  /**
   * Gets a part of a Minecraft player's skin.
   *
   * @param part the part of the skin
   * @param id the id of the player
   * @returns the player's skin part, or null if the player does not exist
   */
  public getPlayerSkinPart(part: string, id: string): Promise<Buffer> {
    return new Promise(async (resolve, reject) => {
      const url = this.playerSkinPartEndpoint.replace("{part}", part).replace("{id}", id);

      const response = await WebRequest.get(url);
      const data = response.data;

      if (response.status !== 200) {
        reject(null);
        return;
      }
      resolve(Buffer.from(data, "utf-8"));
    });
  }
}
