import { MinecraftUtils } from "..";
import WebRequest from "../common/WebRequest";
import { CachedMinecraftServer } from "../types/cache/cachedMinecraftServer";
import { ServerPlatform } from "../types/server/platform";

export default class ServerTools {
  public serverEndpoint = MinecraftUtils.API_ENDPOINT + "/server/{platform}/{hostname}";
  public serverIconEndpoint = MinecraftUtils.API_ENDPOINT + "/server/icon/{hostname}";
  public blockedServerEndpoint = MinecraftUtils.API_ENDPOINT + "/server/blocked/{hostname}";

  /**
   * Gets information about a Minecraft server.
   *
   * @param platform the platform of server
   * @param hostname the hostname of the server
   * @param port the port of the server
   * @returns the server information, or null if the server does not exist
   */
  public getServer(platform: ServerPlatform, hostname: string, port?: 25565): Promise<CachedMinecraftServer> {
    return new Promise(async (resolve, reject) => {
      const ip = port ? `${hostname}:${port}` : hostname;
      const url = this.serverEndpoint.replace("{platform}", platform).replace("{hostname}", ip);

      const response = await WebRequest.get(url);
      const data = response.data;

      if (response.status !== 200) {
        reject(null);
        return;
      }
      resolve(data);
    });
  }

  /**
   * Gets the icon of a Java Minecraft server.
   *
   * @param hostname the hostname of the server
   * @param port the port of the server
   * @returns the server icon, or null if the server does not have an icon
   */
  public getServerIcon(hostname: string, port?: 25565): Promise<Buffer> {
    return new Promise(async (resolve, reject) => {
      const ip = port ? `${hostname}:${port}` : hostname;
      const url = this.serverIconEndpoint.replace("{hostname}", ip);

      const response = await WebRequest.get(url);
      const data = response.data;

      if (response.status !== 200) {
        reject(null);
        return;
      }
      resolve(Buffer.from(data, "utf-8"));
    });
  }

  /**
   * Gets the Mojang blocked status of a Minecraft server.
   *
   * @param hostname the hostname of the server
   * @returns true if the server is blocked, false otherwise
   */
  public getBlockedStatus(hostname: string): Promise<boolean> {
    return new Promise(async (resolve, reject) => {
      const url = this.blockedServerEndpoint.replace("{hostname}", hostname);

      const response = await WebRequest.get(url);
      const data = response.data;

      if (response.status !== 200) {
        reject(null);
        return;
      }
      resolve(data.blocked);
    });
  }
}
